@@ -36,11 +36,7 @@
 #include "../interface.h"
 #include "../cookies.h"
 #include "../prefs.h"
-
-/* Check the Unix98 goodie */
-#ifndef socklen_t
-   #define socklen_t guint32
-#endif
+#include "../misc.h"
 
 /* Used to send a message to the bw's status bar */
 #define BW_MSG(web, root, fmt...)                                     \
@@ -76,7 +72,7 @@ static Klist_t *ValidSocks = NULL; /* Active sockets list. It holds pointers to
                                     * SocketData_t structures. */
 
 static DilloUrl *HTTP_Proxy = NULL;
-
+static gchar *HTTP_Proxy_Auth_base64 = NULL;
 
 /*
  * Initialize proxy vars.
@@ -89,17 +85,41 @@ gint a_Http_init(void)
       HTTP_Proxy = a_Url_new(env_proxy, NULL, 0, 0, 0);
    if (!HTTP_Proxy && prefs.http_proxy)
       HTTP_Proxy = a_Url_dup(prefs.http_proxy);
+
+/*  This allows for storing the proxy password in "user:passwd" format
+ * in dillorc, but as this constitutes a security problem, it was disabled.
+ *
+   if (HTTP_Proxy && prefs.http_proxyuser && strchr(prefs.http_proxyuser, ':'))
+      HTTP_Proxy_Auth_base64 = a_Misc_encode_base64(prefs.http_proxyuser);
+ */
    return 0;
 }
 
+/*
+ * Tell whether the proxy auth is already set (user:password)
+ * Return: 1 Yes, 0 No
+ */
+gint a_Http_proxy_auth(void)
+{
+   return (HTTP_Proxy_Auth_base64 ? 1 : 0);
+}
+
+/*
+ * Activate entered proxy password for HTTP.
+ */
+void a_Http_set_proxy_passwd(gchar *str)
+{
+   gchar *http_proxyauth = g_strconcat(prefs.http_proxyuser, ":", str, NULL);
+   HTTP_Proxy_Auth_base64 = a_Misc_encode_base64(http_proxyauth);
+   g_free(http_proxyauth);
+}
+
 /*
  * Forward declarations
  */
 static void Http_send_query(ChainLink *Info, SocketData_t *S);
 static int Http_connect_socket(ChainLink *Info);
 
-
-
 /*
  * Create and init a new SocketData_t struct, insert into ValidSocks,
  * and return a primary key for it.
@@ -128,12 +148,13 @@ static void Http_socket_free(gint SKey)
 /*
  * Make the http query string
  */
-static char *Http_query(const DilloUrl *url, gboolean use_proxy)
+gchar *a_Http_make_query_str(const DilloUrl *url, gboolean use_proxy)
 {
    gchar *str, *ptr, *cookies;
-   GString *s_port    = g_string_new(""),
-           *query     = g_string_new(""),
-           *full_path = g_string_new("");
+   GString *s_port     = g_string_new(""),
+           *query      = g_string_new(""),
+           *full_path  = g_string_new(""),
+           *proxy_auth = g_string_new("");
 
    /* Sending the default port in the query may cause a 302-answer.  --Jcid */
    if (URL_PORT(url) && URL_PORT(url) != DILLO_URL_HTTP_PORT)
@@ -145,6 +166,9 @@ static char *Http_query(const DilloUrl *url, gboolean use_proxy)
                         (URL_PATH_(url) || URL_QUERY_(url)) ? "" : "/");
       if ((ptr = strrchr(full_path->str, '#')))
          g_string_truncate(full_path, ptr - full_path->str);
+      if (HTTP_Proxy_Auth_base64)
+         g_string_sprintf(proxy_auth, "Proxy-Authorization: Basic %s\r\n",
+                          HTTP_Proxy_Auth_base64);
    } else {
       g_string_sprintfa(full_path, "%s%s%s%s",
                         URL_PATH(url),
@@ -159,6 +183,7 @@ static char *Http_query(const DilloUrl *url, gboolean use_proxy)
          query,
          "POST %s HTTP/1.0\r\n"
          "Host: %s%s\r\n"
+         "%s"
          "User-Agent: Dillo/%s\r\n"
          "Cookie2: $Version=\"1\"\r\n"
          "%s"
@@ -166,7 +191,8 @@ static char *Http_query(const DilloUrl *url, gboolean use_proxy)
          "Content-length: %ld\r\n"
          "\r\n"
          "%s",
-         full_path->str, URL_HOST(url), s_port->str, VERSION, cookies,
+         full_path->str, URL_HOST(url), s_port->str,
+         proxy_auth->str, VERSION, cookies,
          (glong)strlen(URL_DATA(url)),
          URL_DATA(url));
 
@@ -176,6 +202,7 @@ static char *Http_query(const DilloUrl *url, gboolean use_proxy)
          "GET %s HTTP/1.0\r\n"
          "%s"
          "Host: %s%s\r\n"
+         "%s"
          "User-Agent: Dillo/%s\r\n"
          "Cookie2: $Version=\"1\"\r\n"
          "%s"
@@ -183,7 +210,9 @@ static char *Http_query(const DilloUrl *url, gboolean use_proxy)
          full_path->str,
          (URL_FLAGS(url) & URL_E2EReload) ?
             "Cache-Control: no-cache\r\nPragma: no-cache\r\n" : "",
-         URL_HOST(url), s_port->str, VERSION,
+         URL_HOST(url), s_port->str,
+         proxy_auth->str,
+         VERSION,
          cookies);
    }
    g_free(cookies);
@@ -192,6 +221,7 @@ static char *Http_query(const DilloUrl *url, gboolean use_proxy)
    g_string_free(query, FALSE);
    g_string_free(s_port, TRUE);
    g_string_free(full_path, TRUE);
+   g_string_free(proxy_auth, TRUE);
    DEBUG_MSG(4, "Query:\n%s", str);
    return str;
 }
@@ -223,13 +253,11 @@ static gboolean
       g_io_channel_close(S->GioCh);
       g_io_channel_unref(S->GioCh);
       S->GioCh = NULL;
-      if (S->addr_list_iter->next) {
-         S->addr_list_iter = S->addr_list_iter->next;
-         if (Http_connect_socket(Info) < 0) {
-            BW_MSG(S->web, 0, "ERROR: unable to connect to remote host");
-            Http_socket_free(SKey);
-            a_Chain_fcb(OpAbort, Info, NULL, NULL);
-         }
+      S->addr_list_iter = S->addr_list_iter->next;
+      if (!S->addr_list_iter || Http_connect_socket(Info) < 0) {
+         BW_MSG(S->web, 0, "ERROR: unable to connect to remote host");
+         Http_socket_free(SKey);
+         a_Chain_fcb(OpAbort, Info, NULL, NULL);
       }
    } else if ( S->Err ) {
       BW_MSG(S->web, 0, "ERROR: %s", g_strerror(S->Err));
@@ -340,7 +368,7 @@ static void Http_send_query(ChainLink *Info, SocketData_t *S)
    void *link;
 
    /* Create the query */
-   query = Http_query(S->Url, S->use_proxy);
+   query = a_Http_make_query_str(S->Url, S->use_proxy);
 
    /* send query */
    BW_MSG(S->web, 1, "Sending query to %s...", URL_HOST_(S->Url));
@@ -352,7 +380,7 @@ static void Http_send_query(ChainLink *Info, SocketData_t *S)
    a_Chain_bcb(OpSend, Info, io, NULL);
 
    /* Tell the cache to start the receiving CCC for the answer */
-   a_Chain_fcb(OpSend, Info, (void *)S->SockFD, NULL);
+   a_Chain_fcb(OpSend, Info, &S->SockFD, NULL);
 }
 
 /*
@@ -505,7 +533,7 @@ void a_Http_ccc(int Op, int Branch, int Dir, ChainLink *Info,
             /* finished sending the HTTP query */
             if (S) {
                a_Chain_del_link(Info, BCK);
-               a_Chain_fcb(OpEnd, Info, (void *)S->SockFD, (void *)S->Url);
+               a_Chain_fcb(OpEnd, Info, &S->SockFD, (void *)S->Url);
                BW_MSG(S->web, 1, "Query sent, waiting for reply...");
                Http_socket_free(SKey);
             }
@@ -534,5 +562,5 @@ void a_Http_freeall(void)
 {
    a_Klist_free(&ValidSocks);
    a_Url_free(HTTP_Proxy);
+   g_free(HTTP_Proxy_Auth_base64);
 }
-
