@@ -28,7 +28,6 @@ extern "C" {
                                           DW_NEEDS_ALLOCATE)
 #define DW_WIDGET_EXTREMES_CHANGED(wid)  (DW_WIDGET(wid)->flags & \
                                           DW_EXTREMES_CHANGED)
-#define DW_WIDGET_REALIZED(wid)          (DW_WIDGET(wid)->flags & DW_REALIZED)
 
 #define DW_WIDGET_USES_HINTS(wid)        (DW_WIDGET(wid)->flags & \
                                           DW_USES_HINTS)
@@ -48,9 +47,8 @@ typedef enum {
    DW_NEEDS_RESIZE     = 1 << 0,
    DW_NEEDS_ALLOCATE   = 1 << 1,
    DW_EXTREMES_CHANGED = 1 << 2,
-   DW_REALIZED         = 1 << 3,
-   DW_USES_HINTS       = 1 << 4,
-   DW_HAS_CONTENT      = 1 << 5,
+   DW_USES_HINTS       = 1 << 3,
+   DW_HAS_CONTENT      = 1 << 4,
 } DwWidgetFlags;
 
 typedef enum
@@ -266,7 +264,7 @@ struct _DwWidget
 
    /* See p_Dw_widget_set_bg_color(). */
    DwStyleColor *bg_color;
-   
+
    /* See a_Dw_widget_set_button_sensitive(). */
    gboolean button_sensitive;
    gboolean button_sensitive_set;
@@ -300,9 +298,6 @@ struct _DwWidgetClass
                                  DwRectangle *area,
                                  GdkEventExpose *event);
 
-   void (*realize)              (DwWidget *widget);
-   void (*unrealize)            (DwWidget *widget);
-
    gboolean (*button_press_event)   (DwWidget *widget,
                                      gint32 x,
                                      gint32 y,
@@ -330,23 +325,21 @@ struct _DwWidgetClass
 
 GtkType a_Dw_widget_get_type        (void);
 
-void    a_Dw_widget_size_request    (DwWidget *widget,
+void    p_Dw_widget_size_request    (DwWidget *widget,
                                      DwRequisition *requisition);
-void    a_Dw_widget_get_extremes    (DwWidget *widget,
+void    p_Dw_widget_get_extremes    (DwWidget *widget,
                                      DwExtremes *extremes);
-void    a_Dw_widget_size_allocate   (DwWidget *widget,
+void    p_Dw_widget_size_allocate   (DwWidget *widget,
                                      DwAllocation *allocation);
-void    a_Dw_widget_set_width       (DwWidget *widget,
+void    p_Dw_widget_set_width       (DwWidget *widget,
                                      gint32 width);
-void    a_Dw_widget_set_ascent      (DwWidget *widget,
+void    p_Dw_widget_set_ascent      (DwWidget *widget,
                                      gint32 ascent);
-void    a_Dw_widget_set_descent     (DwWidget *widget,
+void    p_Dw_widget_set_descent     (DwWidget *widget,
                                      gint32 descent);
-void    a_Dw_widget_draw            (DwWidget *widget,
+void    p_Dw_widget_draw            (DwWidget *widget,
                                      DwRectangle *area,
                                      GdkEventExpose *event);
-void    a_Dw_widget_realize         (DwWidget *widget);
-void    a_Dw_widget_unrealize       (DwWidget *widget);
 
 void    a_Dw_widget_set_style       (DwWidget *widget,
                                      DwStyle *style);
@@ -398,10 +391,13 @@ void        p_Dw_iterator_highlight_std      (DwIterator *it,
                                               gint from,
                                               gint start,
                                               DwHighlightLayer layer);
-void        p_Dw_iterator_get_allocation_std (DwIterator *it,
-                                              gint start,
-                                              gint end,
-                                              DwAllocation *allocation);
+
+void        p_Dw_iterator_get_allocation_std_only_widgets (DwIterator *it,
+                                                           gint start,
+                                                           gint end,
+                                                           DwAllocation
+                                                           *allocation);
+
 DwIterator* p_Dw_iterator_clone_std_int      (DwIterator *it);
 gint        p_Dw_iterator_compare_std_int    (DwIterator *it1,
                                               DwIterator *it2);
@@ -412,15 +408,19 @@ gchar*      a_Dw_content_html             (DwContent *content);
 gchar*      a_Dw_iterator_text            (DwIterator *it);
 void        a_Dw_widget_print_tree        (DwWidget *widget);
 
-DwIterator* p_Dw_widget_text_iterator     (DwWidget* widget,
-                                           gint32 mask,
-                                           gboolean at_end,
-                                           gchar *text);
-gboolean    p_Dw_iterator_text_next       (DwIterator *it);
-gboolean    p_Dw_iterator_text_prev       (DwIterator *it);
-DwIterator* p_Dw_iterator_text_clone      (DwIterator *it);
-gint        p_Dw_iterator_text_compare    (DwIterator *it1,
-                                           DwIterator *it2);
+DwIterator* p_Dw_widget_text_iterator         (DwWidget* widget,
+                                               gint32 mask,
+                                               gboolean at_end,
+                                               gchar *text);
+gboolean    p_Dw_iterator_text_next           (DwIterator *it);
+gboolean    p_Dw_iterator_text_prev           (DwIterator *it);
+DwIterator* p_Dw_iterator_text_clone          (DwIterator *it);
+gint        p_Dw_iterator_text_compare        (DwIterator *it1,
+                                               DwIterator *it2);
+void        p_Dw_iterator_text_get_allocation (DwIterator *it,
+                                               gint start,
+                                               gint end,
+                                               DwAllocation *allocation);
 
 /* Only for Dw module */
 gint    p_Dw_widget_intersect         (DwWidget *widget,
@@ -443,6 +443,7 @@ gint    Dw_widget_mouse_event       (DwWidget *widget,
                                      gint32 x,
                                      gint32 y,
                                      GdkEvent *event);
+void    Dw_widget_update_cursor       (DwWidget *widget);
 void    p_Dw_widget_queue_draw        (DwWidget *widget);
 void    p_Dw_widget_queue_draw_area   (DwWidget *widget,
                                        gint32 x,
